class Scale:
    # SCALE_FACTOR = 1 units are in mm
    # SCALE_FACTOR = 10 units are in 10 mm
    # SCALE_FACTOR = 100 units are in .1 m
    SCALE_FACTOR = 1
    LENGTH = SCALE_FACTOR
    AREA = SCALE_FACTOR * SCALE_FACTOR
    VOLUME = SCALE_FACTOR * SCALE_FACTOR * SCALE_FACTOR
